;************************************************************************************
; Program to effect shaft encoder reading. Produces Up/Down and Clock signals.      *
; This is a version for the encoder type that has 4 edges per detent.               *
; Uses all 4 I/O pins of a 6-pin PIC10F206, 2 in, 2 out.                            *
; Internal clock is about 4 MHz, so instruction clock is 1 MHz (1 us).              *
;                                                                                   *
; Version that uses retlw type lookup table.                                        *
;                                                                                   *
;************************************************************************************
;    Filename:	    5_8P1.asm                                                       *
;    Started:       19-Dec-2009                                                     *
;    File Version:  Release 2.0                                                     *
;    Release date:  13-Oct-2010                                                     *
;    Author:        K D Brown                                                       *
;    Company:       PLD Designs                                                     *
;************************************************************************************
;    Files required:                                                                *
;      p10F206.inc  Microchip-supplied definition file                              *
;                                                                                   *
;************************************************************************************
	list      p=10F206     ;directive to define processor--any 10F2xx OK
	#include <p10F206.inc> ;processor specific variable definitions
	__CONFIG   _MCLRE_OFF & _CP_OFF & _WDT_OFF

;***** Variable definitions:
;Adresses 0x10 to 0x1F are GP registers in all chips and 0x08 to 0x0F are 
;additional ones in '202/'206 and '222. 
;The variable combo is used as index into the array of 16 possible states
combo          EQU 0x08   ;both previous and present encoder bits, 2 bits each (LSN)
entry          EQU 0x09   ;storage for the present entry
internal_count EQU 0x0A   ;used when more than 1 edge per detent
needed_count   EQU 0x0B   ;debounce counter
old_sample     EQU 0x0C   ;
flags          EQU 0x0D   ;2 flags defined currently, A_edge, B_edge
looked_up      EQU 0x0E   ;a place to put the found action code

;***** flags /bit positions:
CK_OUT    EQU 0  ;in port
U_D_OUT   EQU 1  ;in port
ENC_B     EQU 2  ;in port
ENC_A     EQU 3  ;in port
A_edge    EQU 3  ;in flags
B_edge    EQU 2  ;in flags

;the following 4 are not used as such, but still serve to define positions:
OLD_B     EQU 0  ;in combo
OLD_A     EQU 1  ;in combo
NEW_B     EQU 2  ;in combo, note same position as in port
NEW_A     EQU 3  ;in combo, also same position as in port

;***** constant(s):
NEEDED    EQU 0x20 ;number of samples that have to be the same to pass debounce test

;**********************************************************************
	ORG     0x1FF           ; processor reset vector
	ORG     0x000           ; coding begins here
	;movwf   OSCCAL          ; update register with factory cal value
  ;this is commented out since some combination of using the PICkit2 software "outside" 
  ;of MPLAB and inside it caused the factory programmed values to be over-written with 
  ;FFF. When this is loaded into OSCCAL, it causes the clock, divided by 4, to be output 
  ;on a pin otherwise defined as an input (channel B of the encoder in this project), 
  ;and to run at the fastest possible speed (check this!). The later might be desireable 
  ;for this project but the former most definately is not!
start
    movlw   0x01
    movwf   CMCON0        ;turn off comparator: IMPORTANT!!! Allows GP0 and GP1 as I/O
    movlw   0xD5
    OPTION                ;don't allow the timer to take over GP2, PSA=0, PS2:0=5
    movlw   0x0C          ;zeroes for GP1:GP0: these are the outputs
    TRIS    GPIO          ;make GP0 and GP1 outputs (GP3 always I, want GP2 to be I)
 
;initialize variables:
    clrf    internal_count ;for when there are 4 edges per detent...
    clrf    needed_count  ;the debouncer's counter
  ;init flags so that both old and new match the 2 external bits:
    movf    GPIO,w        ;read in the port
    andlw   0x0C          ;just want two bits from it!
    movwf   old_sample    ;version needed for debounce logic
    movwf   combo
    bcf     STATUS,C
    rrf     combo,f
    bcf     STATUS,C
    rrf     combo,f
    iorwf   combo,f       ;combo <- (combo >>2 ) | w
  ;init flags needed for 4-edge per detent type encoder:
    bcf     flags,A_edge
    bcf     flags,B_edge
       
    bcf     GPIO,CK_OUT   ;set clock's default to Low...
    
loop_start
    movf    GPIO,w        ;read in the port
    andlw   0x0C          ;just want two bits from it!
    movwf   entry         ;store these two bits into a variable
    subwf   old_sample,w  ;is the new value the same as the previous value?
    btfsc   STATUS,Z      ;if subwf result was zero then old and new are (still) the same
    goto    are_same
    ;since samples are different, need to re-start the debounce counter:
    movlw   NEEDED
    movwf   needed_count  ;needed_count <- NEEDED
    ; need to put the new reading into old_reading
    movf    entry,w       ;get back newest reading
    movwf   old_sample    ;now the new is the old
    goto    loop_start
are_same  
    ;the old and new match so decrement the counter if not at zero already:
    movf    needed_count,f ;syntax is basically a test for zero
    btfsc   STATUS,Z ;
    goto    at_end        ;at end of search for N values the same 
    ;otherwise just decrement the counter:
    decf    needed_count,f
    goto    loop_start
at_end
    ;build up old/new switch states, bit order is:  A, B, old_A, old_B:
    movf    entry,w       ;get back newest reading
    ;copy the previous new into the old:
    bcf     STATUS,C
    rrf     combo,f       ;now that we have new data, out with the old...
    bcf     STATUS,C
    rrf     combo,f       ;two bits, making what was new last time, old now.
    movf    combo,w       ;read into working register
    andlw   0x03          ;want the two lsbits only
    
    bcf     GPIO,CK_OUT   ;set clock's default to Low...position helps to extend pulse width
    
    ;combine them:
    iorwf   entry,w       ;w <- w | entry, where w has the two previous bits, now called old
    movwf   combo         ;combo <- w
    ;look up the action for this combination of old and new:
    call    lookup
    movwf   looked_up
    andlw   0xff           ;test for zero
    ;now branch according to code:
    btfsc   STATUS,Z
    goto    loop_start    ;either the same as last time or an error
    ;STATUS,Z was clear so NOT zero, can be any of 6, 9, 5 or A.
    ;lsb tested first
    andlw   0x01          ;is it 1?
    btfsc   STATUS,Z      ;1 & 1 =1, not zero, 1 & 2 = 0, Z set
    goto    is_two
is_one ; i.e. decrement the internal counter and set external DOWN:
    bcf     GPIO,U_D_OUT
    decf    internal_count,f
    goto    edge_detect   ;
is_two ; i.e. increment the internal counter and set external UP:
    bsf     GPIO,U_D_OUT
    incf    internal_count,f
edge_detect
    movf    looked_up,w
    andlw   0x04          ;bit 2 signifies that an A-edge has occurred
    btfsc   STATUS,Z      ;0x04 & 0x04 =1, not zero, 0x04 & 0x08 =0, Z set
    goto    is_A_edge
is_B_edge
    bsf     flags,B_edge
    goto    pulse_maybe
is_A_edge
    bsf     flags,A_edge
pulse_maybe
    movf    internal_count,w ;
    andlw   0x03 ;only every 4th count can cause an external pulse
    btfss   STATUS,Z
    goto    loop_start
    btfss   flags,A_edge
    goto    loop_start
    btfss   flags,B_edge
    goto    loop_start
    ;pulse the outside world counter:
    bsf     GPIO,CK_OUT   ;STATUS,Z was set so internal_count is 0
    ;reset flags needed for 4-edge per detent type encoder
clear_flags
    bcf     flags,A_edge
    bcf     flags,B_edge
    ;bcf     GPIO,CK_OUT
    goto    loop_start    ;back to the start of the loop
    
lookup:
;The codes returned are more generic than needed: bit 0 and bit 1 are 
;redundant and bits 2 and 3 are also, in both cases if and only if the
;returned code is NOT zero. (IE b0=!b1, b2=!b3 when code!=0)
    addwf   PCL,f
    retlw   0
    retlw   6
    retlw   9
    retlw   0
    
    retlw   5
    retlw   0
    retlw   0
    retlw   0x0a

    retlw   0x0a
    retlw   0
    retlw   0
    retlw   5
    
    retlw   0
    retlw   9
    retlw   6
    retlw   0
    
	END